// =======================================================================
//
// Source5.c
// Development for N-Body (Asteroids) Problem
// Earth, Sun and a M Asteroids
// CBP 02-04-22
// Status: Releasse
//
// =======================================================================

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>		// Needed for clock_t
#include <omp.h>
#include "OctaveSupport.h"
#include "nbody.h"

#define DEBUG false

// ------------------- Here's where to change the number of iterations ---
#define NR_ITERATIONS 1000

#define SUN 0
#define EARTH 1
#define ASTS 2

#define MASS_SUN 1.989e30
#define MASS_EARTH 5.97e24
#define MASS_ASTER 3e21
#define ORBIT_RAD_EARTH_SUN 149.6e6
#define ORBIT_RAD_ASTER_SUN (3*ORBIT_RAD_EARTH_SUN)

Body* bodies;

FILE* logfile;

clock_t t0, t1, t2;
double time2;

int main(int argc, char** argv) {

	double dT = 0.01;
	double time = 0;
	int nrItns = NR_ITERATIONS;
	int nrProcs;
	int nrAsteroids, nrBodies;
	int nrRequThreads;
	double dr[2], d[2], dist, force_mag, force_ij[2], tot_force_i[2], dv[2];
	int i, j;

	printf("============== CBP NoC 2022 Multi-Processing Workshops ============\n\n");
	printf("---------------------------N_BODY_PROBLEM -------------------------\n");

	printf("Request to system for coresand threads\n");
	system("WMIC CPU Get NumberofCores,NumberOfLogicalProcessors");

	nrProcs = omp_get_num_procs();
	printf("OMP says nr of threads on this machine = %d\n", nrProcs);
	printf("Input number of threads you want  ");
	scanf_s("%d", &nrRequThreads);

	printf("Input number of asteroids you want  ");
	scanf_s("%d", &nrAsteroids);
	nrBodies = nrAsteroids + 2;

	omp_set_dynamic(0);
	omp_set_num_threads(nrRequThreads);

	printf("\nNr asteroids = %d, Nr bodies = %d\n\n", nrAsteroids, nrBodies);
	printf("Nr iterations set = %d\n\n", nrItns);

	fopen_s(&logfile, "log.m", "w");
	writeOctaveHead(logfile);

	bodies = init_bodies(nrBodies, nrAsteroids);

	// Start the timer ---------------------------------
	t0 = clock();

	// Main Computational loop -------------------------
	for (int k = 0; k < nrItns; k++) {
		writeDataToOctave(logfile, time, bodies, nrBodies);

		// Here's where to build up your code ----------



		


		// End of your code above here -----------------
		time += dT;
	}

	// End the timer here ------------------------------
	t2 = clock();
	time2 = (double)(t2)-(double)(t0);
	printf("Time for the run = %e millisec (%f millisec)\n", time2,time2);

	writeOctaveTail(logfile, nrBodies);

}

Body* init_bodies(unsigned int num_bodies, int num_asteroids) {
	int i;
	double n = num_bodies;
	double r, v, angleDes, theta, period, maxPeriod, maxR;
	Body* bodies = (Body*)malloc(num_bodies * sizeof(Body));
	maxR = 0;
	maxPeriod = 0;

	// Sun --------------------------
	bodies[SUN].mass = 1.0;
	bodies[SUN].pos[X] = 0;
	bodies[SUN].pos[Y] = 0;
	bodies[SUN].vel[X] = 0.0;
	bodies[SUN].vel[Y] = 0.0;

	// Earth ------------------------
	bodies[EARTH].mass = MASS_EARTH / MASS_SUN;
	r = ORBIT_RAD_EARTH_SUN / ORBIT_RAD_EARTH_SUN;
	if (r > maxR) maxR = r;
	v = sqrt(bodies[SUN].mass / r);
	angleDes = 45.0;
	theta = angleDes * M_PI / 180;
	period = 2 * M_PI * r / v;
	if (period > maxPeriod) maxPeriod = period;
	bodies[EARTH].pos[X] =  r * cos(theta);
	bodies[EARTH].pos[Y] =  r * sin(theta);
	bodies[EARTH].vel[X] = -v * sin(theta);
	bodies[EARTH].vel[Y] =  v * cos(theta);
	printf("Earth: r = %e, v = %e\n", r, v);
	printf("Earth: m = %e, x = %e, y = %e, vX = %e, vY = %e\n", bodies[EARTH].mass, bodies[EARTH].pos[X], bodies[EARTH].pos[Y], 
		              bodies[EARTH].vel[X], bodies[EARTH].vel[Y]);

	for (i = 0; i < num_asteroids; i++) {
		bodies[ASTS + i].mass = MASS_ASTER / MASS_SUN;
		r = ORBIT_RAD_ASTER_SUN / ORBIT_RAD_EARTH_SUN;
		double bump = ((double)rand() / (double)RAND_MAX);
		//printf("bump%d = %f\n", i, bump);
		r = r * (1 + bump / 10.0);
		if (r > maxR) maxR = r;
		v = sqrt(bodies[SUN].mass / r);
		angleDes = 360 / (i + 1);
		theta = angleDes * M_PI / 180;
		//printf("theta%d = %f\n", i, theta);
		period = 2 * M_PI * r / v;
		if (period > maxPeriod) maxPeriod = period;
		bodies[ASTS + i].pos[X] = r * cos(theta);
		bodies[ASTS + i].pos[Y] = r * sin(theta);
		bodies[ASTS + i].vel[X] = -v * sin(theta);
		bodies[ASTS + i].vel[Y] = v * cos(theta);

		//printf("Ast %d   r = %e, v = %e\n", i, r, v);
		//printf("Ast %d   m = %e, x = %e, y = %e, vX = %e, vY = %e\n", i,
		//		bodies[ASTS + i].mass, bodies[ASTS + i].pos[X], bodies[ASTS + i].pos[Y], bodies[ASTS + i].vel[X], bodies[ASTS + i].vel[Y]);


	}
	return bodies;
}
